package main

import "fmt"

type Fly interface {
	Fly(name string) bool
}

type Swim interface {
	Swim(name string) bool
}

type Feature interface {
	//Fly(name string) bool
	//Swim(name string) bool
	// 匿名嵌入，效果等同于上面定义
	Fly
	Swim

	// 接口的其他方法
	do(name string)
	clean()
}

// Duck 实现了 Feature、Swim、Fly 3个接口
type Duck struct {
}

func (d Duck) Fly(name string) bool {
	fmt.Println(name, "can fly")
	return true
}

func (d Duck) Swim(name string) bool {
	fmt.Println(name, "can swim")
	return true
}

func (d Duck) do(name string) {
	d.Fly(name)
	d.Swim(name)
}

func (d Duck) clean() {
	fmt.Println("clean")
}

func main() {
	// 匿名嵌套
	d := Duck{}
	// 游玩好后，延迟清理
	defer d.clean()

	var fly Fly = d
	var swim Swim = d
	var feature Feature = d

	fmt.Printf("%T %#v %T %#v %T %#v\n", fly, fly, swim, swim, feature, feature)
	fly.Fly("tanglaoya")
	swim.Swim("tanglaoya")
	feature.do("tanglaoya")

	// 匿名接口
	var chicken interface {
		Fly(name string) bool
	}

	chicken = fly
	chicken.Fly("xiaoji")

	/* Output:
	   main.Duck main.Duck{} main.Duck main.Duck{} main.Duck main.Duck{}
	   tanglaoya can fly
	   tanglaoya can swim
	   tanglaoya can fly
	   tanglaoya can swim
	   xiaoji can fly
	   clean
	*/
}
